@@ -4,12 +4,12 @@
 import datetime as dt
 import hashlib
 import json
-from typing import List
+import struct
+from typing import List, Tuple
 
 import pydantic
-import ueosio
 
-from . import types
+from . import types, utils
 from .net import Net
 
 
@@ -177,6 +177,23 @@ class LinkedAction(Action):
         return bytes_
 
 
+def _endian_reverse_u32(i: int) -> int:
+    i = i & 0xFFFFFFFF
+    r = (((i >> 0x18) & 0xFF)) | (((i >> 0x10) & 0xFF) << 0x08) | (((i >> 0x08) & 0xFF) << 0x10) | (((i) & 0xFF) << 0x18)  # NOQA BLK100, E501
+    return r
+
+
+def _get_tapos_info(block_id: str) -> Tuple[int]:
+    block_id_bin = bytes.fromhex(block_id)
+
+    hash0 = struct.unpack("<Q", block_id_bin[0:8])[0]
+    hash1 = struct.unpack("<Q", block_id_bin[8:16])[0]
+
+    ref_block_num = _endian_reverse_u32(hash0) & 0xFFFF
+    ref_block_prefix = hash1 & 0xFFFFFFFF
+    return ref_block_num, ref_block_prefix
+
+
 class Transaction(pydantic.BaseModel):
     """
     Raw Transaction. It can't be sent to the blockchain.
@@ -207,9 +224,7 @@ class Transaction(pydantic.BaseModel):
         block_id = net_info["last_irreversible_block_id"]
         chain_id = net_info["chain_id"]
 
-        ref_block_num, ref_block_prefix = ueosio.get_tapos_info(
-            block_id=block_id
-        )
+        ref_block_num, ref_block_prefix = _get_tapos_info(block_id=block_id)
         expiration = dt.datetime.utcnow() + dt.timedelta(
             seconds=self.expiration_delay_sec
         )
@@ -286,7 +301,7 @@ class LinkedTransaction(Transaction):
         zero_bytes = bytes.fromhex("0" * 64)
         bytes_ = chain_bytes + trans_bytes + zero_bytes
 
-        signature = sign_bytes(bytes_, key)
+        signature = utils.sign_bytes(bytes_=bytes_, key=key)
         signs.append(signature)
         trans = SignedTransaction(
             net=self.net,
@@ -304,25 +319,6 @@ class LinkedTransaction(Transaction):
         return trans
 
 
-def sign_bytes(bytes_: bytes, key: str) -> str:
-    nonce = 0
-    sha256 = hashlib.sha256()
-    sha256.update(bytes_)
-    while True:
-        v, r, s = ueosio.utils.ecdsa_raw_sign_nonce(
-            sha256.digest(), key, nonce
-        )
-        if ueosio.utils.is_canonical(r, s):
-            signature = "00%02x%064x%064x" % (v, r, s)
-            break
-        nonce += 1
-
-    ds = ueosio.DataStream(bytes.fromhex(signature))
-    signature = ds.unpack_signature()
-
-    return signature
-
-
 class SignedTransaction(LinkedTransaction):
     """
     Signed transaction. You can send it to the blockchain.
